# Copyright (c) 2019  University of Houston Networking Lab

# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from __future__ import absolute_import

import struct

from ..types import EthernetMAC, IPv4Address

class HRD(object):
  ETHERNET = 1
  IEEE802 = 6

def parse (pk):
  """Parse ARP data out of previously-parsed Ethernet packet"""

  if not isinstance(pk, dict):
    raise ValueError()

  (pk["hrd"], pk["pro"], pk["hln"], pk["pln"], pk["op"]) = struct.unpack(">HHBBH", pk["data"][:8])

  if pk["hrd"] in set([HRD.ETHERNET, HRD.IEEE802]):
    fmt = ">%ds%ds%ds%ds" % (pk["hln"], pk["pln"], pk["hln"], pk["pln"])
    (sha, spa, tha, tpa) = struct.unpack(fmt, pk["data"][8:8+struct.calcsize(fmt)])
    pk["sha"] = EthernetMAC(sha)
    pk["tha"] = EthernetMAC(tha)
    pk["spa"] = IPv4Address(spa)
    pk["tpa"] = IPv4Address(tpa)

  return pk

