# Copyright (c) 2019  University of Houston Networking Lab

# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from __future__ import absolute_import

import struct

from ..types import EthernetMAC

def parse (data):
  """Parse ethernet header fields out of packet byte stream"""

  pf = struct.unpack(">6s6sH", data[:14])
  tobj = {"src" : EthernetMAC(pf[1]), "dst" : EthernetMAC(pf[0]), "type" : pf[2],
          "tag" : None, "data" : None}
  if pf[2] == 0x8100:
    parts = struct.unpack(">HH", data[14:18])
    tobj["tag"] = parts[0]
    tobj["type"] = parts[1]
    tobj["data"] = data[18:]
  else:
    tobj["data"] = data[14:]

  return tobj
