# Copyright (c) 2019-2021  University of Houston Networking Lab 
# import pytest

from uhnet.graph import packing

def test_VertexBase ():
  v1 = packing.VertexBase(10, name='v1')
  v2 = packing.VertexBase(5, name='v2')
  assert(v1.available_capacity() == 10)
  assert(v1.getLoad() == 0.0)

  v1.addLoad(v2)
  assert(v1.available_capacity() == 5)
  assert(v1.getLoad() == 0.5)
#  with pytest.raises(packing.VertexAlreadyLoadedError):
#    v1.addLoad(v2)

  v1.delLoad(v2)
  with pytest.raises(packing.UnknownVertexError):
    v1.delLoad(v2)  

def test_vertexcapacity ():
  v1 = packing.VertexBase(10, name='v1')
  v2 = packing.VertexBase(5, name='v2')
  v3 = packing.VertexBase(5, name='v3')
  v1.applyCapacityLimit(0.8)
  assert(v1.available_capacity() == 8)
  assert(v1.getLoad() == 0.0)

  v1.addLoad(v2)
  assert(v1.available_capacity() == 3)
  assert(v1.getLoad() == 0.5)
  with pytest.raises(packing.CapacityLimitExceededError):
    v1.addLoad(v3)

def test_neighbors ():
  v1 = packing.VertexBase(10, name='v1')
  v2 = packing.VertexBase(5, name='v2')
  e = packing.EdgeBase()
  e.connect(v1, v2)
  assert(v2 in v1.getNeighbors())
  assert(v1 in v2.getNeighbors())

def test_getEdgelessCopy():
  v1 = packing.VertexBase(10, name='v1')
  v2 = packing.VertexBase(5, name='v2')
  e = packing.EdgeBase()
  e.connect(v1, v2)
  v3 = v1.getEdgelessCopy()
  assert(len(v3.getNeighbors()) == 0) 
  assert(not v3._edges)

def test_getNodelessCopy():
  v1 = packing.VertexBase(10, name='v1')
  v2 = packing.VertexBase(5, name='v2')
  e = packing.EdgeBase()
  e.connect(v1, v2)
  new_e = e.getNodelessCopy()
  assert(new_e._vertex_a is None)
  assert(new_e._vertex_b is None)

def test_connectVertices():
  v1 = packing.VertexBase(10, name='v1')
  v2 = packing.VertexBase(5, name='v2')
  v3 = packing.VertexBase(5, name='v3')
  v4 = packing.VertexBase(5, name='v4')
  e = packing.EdgeBase()
  e.connect(v1, v2)
  assert(e._vertex_a is not None)
  assert(e._vertex_b is not None)
  with pytest.raises(packing.MaxVerticesExceededError):
    e.connect(v3, v4)

def test_addVertices():
  v1 = packing.VertexBase(10, name='v1')
  v2 = packing.VertexBase(5, name='v2')
  v3 = packing.VertexBase(5, name='v3')
  e = packing.EdgeBase()
  e.addVertex(v1)
  assert(e._vertex_a is not None)
  e.addVertex(v2)
  assert(e._vertex_b is not None)
  with pytest.raises(packing.MaxVerticesExceededError):
    e.addVertex(v3)
 
