# Copyright (c) 2019-2021 University of Houston Networking Lab

import pcapy
import pytest

import uhnet.pcap.arp
import uhnet.pcap.eth
import uhnet.types

### Fixtures

@pytest.fixture(scope="module")
def vlan_packets ():
  r = pcapy.open_offline("tests/data/vlan.pcap")
  hdr,pkt = r.next()
  pkt_list = []
  while hdr is not None:
    pkt_list.append(pkt)
    hdr,pkt = r.next()
  return pkt_list

@pytest.fixture(scope="module")
def arp_packets ():
  r = pcapy.open_offline("tests/data/arp.pcap")
  hdr,pkt = r.next()
  pkt_list = []
  while hdr is not None:
    pkt_list.append(pkt)
    hdr,pkt = r.next()
  return pkt_list

### Basic Ethernet

def test_eth1 (vlan_packets):
  pd = uhnet.pcap.eth.parse(vlan_packets[0])
  assert(pd["tag"] == None)
  assert(pd["src"] == uhnet.types.EthernetMAC("2a:04:7d:fc:d2:7e"))
  assert(pd["dst"] == uhnet.types.EthernetMAC("aa:bb:cc:dd:ee:a9"))
  assert(pd["type"] == 0x820)

def test_eth2 (vlan_packets):
  pd = uhnet.pcap.eth.parse(vlan_packets[1])
  assert(pd["tag"] == 1694)
  assert(pd["src"] == uhnet.types.EthernetMAC("1a:63:40:a3:1d:bf"))
  assert(pd["dst"] == uhnet.types.EthernetMAC("aa:bb:cc:dd:ee:19"))
  assert(pd["type"] == 0x820)

### ARP

def test_arp1 (arp_packets):
  pd = uhnet.pcap.eth.parse(arp_packets[0])
  uhnet.pcap.arp.parse(pd)
  assert(pd["type"] == 0x806)
  assert(pd["hrd"] == uhnet.pcap.arp.HRD.ETHERNET)
  assert(pd["sha"] == uhnet.types.EthernetMAC("5e:3a:52:d0:53:ef"))
  assert(pd["spa"] == uhnet.types.IPv4Address("219.195.248.23"))
  assert(pd["tha"] == uhnet.types.EthernetMAC(0))
  assert(pd["tpa"] == uhnet.types.IPv4Address("219.195.250.66"))

def test_arp_badinput1 (arp_packets):
  with pytest.raises(ValueError):
    uhnet.pcap.arp.parse("hi mom")

def test_arp_badinput2 (arp_packets):
  with pytest.raises(ValueError):
    uhnet.pcap.arp.parse(arp_packets[0])
